@@ -35,16 +35,51 @@
 #include <string.h>
 #include <errno.h>
 #include <unistd.h>
+#include <limits.h>
 
 #include "../lib.h"
 #include "../parameters/parameters.h"
 #include "../display.h"
 
+static int intel_cpu_models[] = {
+	0x1A,	/* Core i7, Xeon 5500 series */
+	0x1E,	/* Core i7 and i5 Processor - Lynnfield Jasper Forest */
+	0x1F,	/* Core i7 and i5 Processor - Nehalem */
+	0x2E,	/* Nehalem-EX Xeon */
+	0x2F,	/* Westmere-EX Xeon */
+	0x25,	/* Westmere */
+	0x27,	/* Medfield Atom*/
+	0x2C,	/* Westmere */
+	0x2A,	/* SNB */
+	0x2D,	/* SNB Xeon */
+	0x37,	/* BYT-M */
+	0x3A,	/* IVB */
+	0x3C,
+	0x3D,	/* Broadwell */
+	0x3E,	/* IVB Xeon */
+	0x3F,	/* HSX */
+	0x45,	/* HSW-ULT */
+	0x46,	/* HSW */
+	0x47,	/* BDW-H */
+	0x4C,	/* BSW */
+	0x4D,	/* AVN */
+	0x4F,	/* BDX */
+	0x4E,	/* SKY */
+	0x5E,	/* SKY */
+	0x56,	/* BDX-DE */
+	0	/* last entry must be zero */
+};
 
-int has_c2c7_res;
-int has_c8c9c10_res;
+int is_supported_intel_cpu(int model)
+{
+	int i;
 
+	for (i = 0; intel_cpu_models[i] != 0; i++)
+		if (model == intel_cpu_models[i])
+			return 1;
 
+	return 0;
+}
 
 static uint64_t get_msr(int cpu, uint64_t offset)
 {
@@ -62,30 +97,85 @@ static uint64_t get_msr(int cpu, uint64_t offset)
 	return msr;
 }
 
+intel_util::intel_util()
+{
+	byt_ahci_support=0;
+}
+
+void intel_util::byt_has_ahci()
+{
+	dir = opendir("/sys/bus/pci/devices/0000:00:13.0");
+        if (!dir)
+                byt_ahci_support=0;
+	else
+		byt_ahci_support=1;
+        closedir(dir);
+}
+
+int intel_util::get_byt_ahci_support()
+{
+	return byt_ahci_support;
+}
+
+nhm_core::nhm_core(int model)
+{
+	has_c7_res = 0;
+
+	switch(model) {
+		case 0x2A:	/* SNB */
+		case 0x2D:	/* SNB Xeon */
+		case 0x3A:      /* IVB */
+		case 0x3C:
+		case 0x3E:      /* IVB Xeon */
+		case 0x45:	/* HSW-ULT */
+		case 0x4E:	/* SKY */
+		case 0x5E:	/* SKY */
+		case 0x3D:	/* Intel Next Generation */
+			has_c7_res = 1;
+	}
+
+	has_c3_res = 1;
+	has_c1_res = 0;
+
+	switch (model) {
+		case 0x37:	/* BYT-M does not support C3/C4 */
+		case 0x4C:	/* BSW does not support C3 */
+			has_c3_res = 0;
+			has_c1_res = 1;
+	}
+
+}
+
 void nhm_core::measurement_start(void)
 {
 	ifstream file;
-	char filename[4096];
+	char filename[PATH_MAX];
 
 	/* the abstract function needs to be first since it clears all state */
 	abstract_cpu::measurement_start();
 
 	last_stamp = 0;
 
-	c3_before    = get_msr(first_cpu, MSR_CORE_C3_RESIDENCY);
+	if (this->has_c1_res)
+		c1_before = get_msr(first_cpu, MSR_CORE_C1_RESIDENCY);
+	if (this->has_c3_res)
+		c3_before    = get_msr(first_cpu, MSR_CORE_C3_RESIDENCY);
 	c6_before    = get_msr(first_cpu, MSR_CORE_C6_RESIDENCY);
-	if (has_c2c7_res)
+	if (this->has_c7_res)
 		c7_before    = get_msr(first_cpu, MSR_CORE_C7_RESIDENCY);
 	tsc_before   = get_msr(first_cpu, MSR_TSC);
 
-	insert_cstate("core c3", "C3 (cc3)", 0, c3_before, 1);
+	if (this->has_c1_res)
+		insert_cstate("core c1", "C1 (cc1)", 0, c1_before, 1);
+	if (this->has_c3_res)
+		insert_cstate("core c3", "C3 (cc3)", 0, c3_before, 1);
 	insert_cstate("core c6", "C6 (cc6)", 0, c6_before, 1);
-	if (has_c2c7_res) {
+	if (this->has_c7_res) {
 		insert_cstate("core c7", "C7 (cc7)", 0, c7_before, 1);
 	}
 
 
-	sprintf(filename, "/sys/devices/system/cpu/cpu%i/cpufreq/stats/time_in_state", first_cpu);
+	snprintf(filename, PATH_MAX, "/sys/devices/system/cpu/cpu%i/cpufreq/stats/time_in_state", first_cpu);
 
 	file.open(filename, ios::in);
 
@@ -110,17 +200,21 @@ void nhm_core::measurement_end(void)
 	uint64_t time_delta;
 	double ratio;
 
-	c3_after    = get_msr(first_cpu, MSR_CORE_C3_RESIDENCY);
+	if (this->has_c1_res)
+		c1_after = get_msr(first_cpu, MSR_CORE_C1_RESIDENCY);
+	if (this->has_c3_res)
+		c3_after    = get_msr(first_cpu, MSR_CORE_C3_RESIDENCY);
 	c6_after    = get_msr(first_cpu, MSR_CORE_C6_RESIDENCY);
-	if (has_c2c7_res)
+	if (this->has_c7_res)
 		c7_after    = get_msr(first_cpu, MSR_CORE_C7_RESIDENCY);
 	tsc_after   = get_msr(first_cpu, MSR_TSC);
 
-
-
-	finalize_cstate("core c3", 0, c3_after, 1);
+	if (this->has_c1_res)
+		finalize_cstate("core c1", 0, c1_after, 1);
+	if (this->has_c3_res)
+		finalize_cstate("core c3", 0, c3_after, 1);
 	finalize_cstate("core c6", 0, c6_after, 1);
-	if (has_c2c7_res)
+	if (this->has_c7_res)
 		finalize_cstate("core c7", 0, c7_after, 1);
 
 	gettimeofday(&stamp_after, NULL);
@@ -195,6 +289,57 @@ char * nhm_core::fill_pstate_line(int line_nr, char *buffer)
 	return buffer;
 }
 
+nhm_package::nhm_package(int model)
+{
+	has_c8c9c10_res = 0;
+	has_c2c6_res = 0;
+	has_c7_res = 0;
+	has_c6c_res = 0;
+
+	switch(model) {
+		case 0x2A:	/* SNB */
+		case 0x2D:	/* SNB Xeon */
+		case 0x3A:      /* IVB */
+		case 0x3C:
+		case 0x3E:      /* IVB Xeon */
+		case 0x45:	/* HSW-ULT */
+		case 0x4E:	/* SKY */
+		case 0x5E:	/* SKY */
+		case 0x3D:	/* Intel Next Generation */
+			has_c2c6_res=1;
+			has_c7_res = 1;
+	}
+
+	has_c3_res = 1;
+
+	switch(model) {
+		/* BYT-M doesn't have C3 or C7 */
+		/* BYT-T doesn't have C3 but it has C7 */
+		case 0x37:
+			has_c2c6_res=1;
+			this->byt_has_ahci();
+			if ((this->get_byt_ahci_support()) == 0)
+				has_c7_res = 1;/*BYT-T PC7 <- S0iX*/
+			else
+				has_c7_res = 0;
+			break;
+		case 0x4C: /* BSW doesn't have C3 */
+			has_c3_res = 0;
+			has_c6c_res = 1; /* BSW only exposes package C6 */
+			break;
+	}
+
+	/*Has C8/9/10*/
+	switch(model) {
+		case 0x45: /*HSW*/
+		case 0x3D:
+		case 0x4E:
+		case 0x5E:
+			has_c8c9c10_res = 1;
+			break;
+	}
+}
+
 char * nhm_package::fill_pstate_line(int line_nr, char *buffer)
 {
 	buffer[0] = 0;
@@ -228,27 +373,38 @@ void nhm_package::measurement_start(void)
 
 	last_stamp = 0;
 
-	if (has_c2c7_res)
+	if (this->has_c2c6_res)
 		c2_before    = get_msr(number, MSR_PKG_C2_RESIDENCY);
-	c3_before    = get_msr(number, MSR_PKG_C3_RESIDENCY);
-	c6_before    = get_msr(number, MSR_PKG_C6_RESIDENCY);
-	if (has_c2c7_res)
+
+	if (this->has_c3_res)
+		c3_before    = get_msr(number, MSR_PKG_C3_RESIDENCY);
+
+	/*
+	 * Hack for Braswell where C7 MSR is actually BSW C6
+	 */
+	if (this->has_c6c_res)
+		c6_before    = get_msr(number, MSR_PKG_C7_RESIDENCY);
+	else
+		c6_before    = get_msr(number, MSR_PKG_C6_RESIDENCY);
+
+	if (this->has_c7_res)
 		c7_before    = get_msr(number, MSR_PKG_C7_RESIDENCY);
-	if (has_c8c9c10_res) {
+	if (this->has_c8c9c10_res) {
 		c8_before    = get_msr(number, MSR_PKG_C8_RESIDENCY);
 		c9_before    = get_msr(number, MSR_PKG_C9_RESIDENCY);
 		c10_before    = get_msr(number, MSR_PKG_C10_RESIDENCY);
 	}
 	tsc_before   = get_msr(first_cpu, MSR_TSC);
 
-	if (has_c2c7_res)
+	if (this->has_c2c6_res)
 		insert_cstate("pkg c2", "C2 (pc2)", 0, c2_before, 1);
 
-	insert_cstate("pkg c3", "C3 (pc3)", 0, c3_before, 1);
+	if (this->has_c3_res)
+		insert_cstate("pkg c3", "C3 (pc3)", 0, c3_before, 1);
 	insert_cstate("pkg c6", "C6 (pc6)", 0, c6_before, 1);
-	if (has_c2c7_res)
+	if (this->has_c7_res)
 		insert_cstate("pkg c7", "C7 (pc7)", 0, c7_before, 1);
-	if (has_c8c9c10_res) {
+	if (this->has_c8c9c10_res) {
 		insert_cstate("pkg c8", "C8 (pc8)", 0, c8_before, 1);
 		insert_cstate("pkg c9", "C9 (pc9)", 0, c9_before, 1);
 		insert_cstate("pkg c10", "C10 (pc10)", 0, c10_before, 1);
@@ -266,11 +422,18 @@ void nhm_package::measurement_end(void)
 			children[i]->wiggle();
 
 
-	if (has_c2c7_res)
+	if (this->has_c2c6_res)
 		c2_after    = get_msr(number, MSR_PKG_C2_RESIDENCY);
-	c3_after    = get_msr(number, MSR_PKG_C3_RESIDENCY);
-	c6_after    = get_msr(number, MSR_PKG_C6_RESIDENCY);
-	if (has_c2c7_res)
+
+	if (this->has_c3_res)
+		c3_after    = get_msr(number, MSR_PKG_C3_RESIDENCY);
+
+	if (this->has_c6c_res)
+		c6_after    = get_msr(number, MSR_PKG_C7_RESIDENCY);
+	else
+		c6_after    = get_msr(number, MSR_PKG_C6_RESIDENCY);
+
+	if (this->has_c7_res)
 		c7_after    = get_msr(number, MSR_PKG_C7_RESIDENCY);
 	if (has_c8c9c10_res) {
 		c8_after = get_msr(number, MSR_PKG_C8_RESIDENCY);
@@ -284,11 +447,13 @@ void nhm_package::measurement_end(void)
 	time_factor = 1000000.0 * (stamp_after.tv_sec - stamp_before.tv_sec) + stamp_after.tv_usec - stamp_before.tv_usec;
 
 
-	if (has_c2c7_res)
+	if (this->has_c2c6_res)
 		finalize_cstate("pkg c2", 0, c2_after, 1);
-	finalize_cstate("pkg c3", 0, c3_after, 1);
+
+	if (this->has_c3_res)
+		finalize_cstate("pkg c3", 0, c3_after, 1);
 	finalize_cstate("pkg c6", 0, c6_after, 1);
-	if (has_c2c7_res)
+	if (this->has_c7_res)
 		finalize_cstate("pkg c7", 0, c7_after, 1);
 	if (has_c8c9c10_res) {
 		finalize_cstate("pkg c8", 0, c8_after, 1);
@@ -340,7 +505,7 @@ void nhm_package::measurement_end(void)
 void nhm_cpu::measurement_start(void)
 {
 	ifstream file;
-	char filename[4096];
+	char filename[PATH_MAX];
 
 	cpu_linux::measurement_start();
 
@@ -352,7 +517,7 @@ void nhm_cpu::measurement_start(void)
 
 	insert_cstate("active", _("C0 active"), 0, aperf_before, 1);
 
-	sprintf(filename, "/sys/devices/system/cpu/cpu%i/cpufreq/stats/time_in_state", first_cpu);
+	snprintf(filename, PATH_MAX, "/sys/devices/system/cpu/cpu%i/cpufreq/stats/time_in_state", first_cpu);
 
 	file.open(filename, ios::in);
 
@@ -405,7 +570,6 @@ void nhm_cpu::measurement_end(void)
 
 }
 
-
 char * nhm_cpu::fill_pstate_name(int line_nr, char *buffer)
 {
 	if (line_nr == LEVEL_C0) {
