@@ -27,10 +27,8 @@
 
 #include <stdio.h>
 #include <sys/types.h>
-#include <dirent.h>
 #include <unistd.h>
 
-
 using namespace std;
 
 #include "device.h"
@@ -42,7 +40,7 @@ using namespace std;
 #include <string.h>
 #include <unistd.h>
 
-alsa::alsa(char *_name, char *path): device()
+alsa::alsa(const char *_name, const char *path): device()
 {
 	ifstream file;
 
@@ -55,47 +53,47 @@ alsa::alsa(char *_name, char *path): device()
 	start_inactive = 0;
 	strncpy(sysfs_path, path, sizeof(sysfs_path));
 
-	sprintf(devname, "alsa:%s", _name);
-	sprintf(humanname, "alsa:%s", _name);
+	snprintf(devname, 4096, "alsa:%s", _name);
+	snprintf(humanname, 4096, "alsa:%s", _name);
 	strncpy(name, devname, sizeof(name));
 	rindex = get_result_index(name);
 
 	guilty[0] = 0;
 	model[0] = 0;
 	vendor[0] = 0;
-	sprintf(devname, "%s/modelname", path);
+	snprintf(devname, 4096, "%s/modelname", path);
 	file.open(devname);
 	if (file) {
 		file.getline(model, 4096);
 		file.close();
 	}
-	sprintf(devname, "%s/vendor_name", path);
+	snprintf(devname, 4096, "%s/vendor_name", path);
 	file.open(devname);
 	if (file) {
 		file.getline(vendor, 4096);
 		file.close();
 	}
 	if (strlen(model) && strlen(vendor))
-		sprintf(humanname, _("Audio codec %s: %s (%s)"), name, model, vendor);
+		snprintf(humanname, 4096, _("Audio codec %s: %s (%s)"), name, model, vendor);
 	else if (strlen(model))
-		sprintf(humanname, _("Audio codec %s: %s"), _name, model);
+		snprintf(humanname, 4096, _("Audio codec %s: %s"), _name, model);
 	else if (strlen(vendor))
-		sprintf(humanname, _("Audio codec %s: %s"), _name, vendor);
+		snprintf(humanname, 4096, _("Audio codec %s: %s"), _name, vendor);
 }
 
 void alsa::start_measurement(void)
 {
-	char filename[4096];
+	char filename[PATH_MAX];
 	ifstream file;
 
-	sprintf(filename, "%s/power_off_acct", sysfs_path);
+	snprintf(filename, PATH_MAX, "%s/power_off_acct", sysfs_path);
 	try {
 		file.open(filename, ios::in);
 		if (file) {
 			file >> start_inactive;
 		}
 		file.close();
-		sprintf(filename, "%s/power_on_acct", sysfs_path);
+		snprintf(filename, PATH_MAX, "%s/power_on_acct", sysfs_path);
 		file.open(filename, ios::in);
 
 		if (file) {
@@ -110,18 +108,18 @@ void alsa::start_measurement(void)
 
 void alsa::end_measurement(void)
 {
-	char filename[4096];
+	char filename[PATH_MAX];
 	ifstream file;
 	double p;
 
-	sprintf(filename, "%s/power_off_acct", sysfs_path);
+	snprintf(filename, PATH_MAX, "%s/power_off_acct", sysfs_path);
 	try {
 		file.open(filename, ios::in);
 		if (file) {
 			file >> end_inactive;
 		}
 		file.close();
-		sprintf(filename, "%s/power_on_acct", sysfs_path);
+		snprintf(filename, PATH_MAX, "%s/power_on_acct", sysfs_path);
 		file.open(filename, ios::in);
 
 		if (file) {
@@ -152,39 +150,28 @@ const char * alsa::device_name(void)
 	return name;
 }
 
-void create_all_alsa(void)
+static void create_all_alsa_callback(const char *d_name)
 {
-	struct dirent *entry;
-	DIR *dir;
-	char filename[4096];
+	char filename[PATH_MAX];
+	class alsa *bl;
 
-	dir = opendir("/sys/class/sound/card0/");
-	if (!dir)
+	if (strncmp(d_name, "hwC", 3) != 0)
 		return;
-	while (1) {
-		class alsa *bl;
-		ofstream file;
-		entry = readdir(dir);
-		if (!entry)
-			break;
-		if (strncmp(entry->d_name, "hwC", 3) != 0)
-			continue;
-		sprintf(filename, "/sys/class/sound/card0/%s/power_on_acct", entry->d_name);
 
-		if (access(filename, R_OK) != 0)
-			continue;
-
-		sprintf(filename, "/sys/class/sound/card0/%s", entry->d_name);
-
-		bl = new class alsa(entry->d_name, filename);
-		all_devices.push_back(bl);
-		register_parameter("alsa-codec-power", 0.5);
-	}
-	closedir(dir);
+	snprintf(filename, PATH_MAX, "/sys/class/sound/card0/%s/power_on_acct", d_name);
+	if (access(filename, R_OK) != 0)
+		return;
 
+	snprintf(filename, PATH_MAX, "/sys/class/sound/card0/%s", d_name);
+	bl = new class alsa(d_name, filename);
+	all_devices.push_back(bl);
+	register_parameter("alsa-codec-power", 0.5);
 }
 
-
+void create_all_alsa(void)
+{
+	process_directory("/sys/class/sound/card0/", create_all_alsa_callback);
+}
 
 double alsa::power_usage(struct result_bundle *result, struct parameter_bundle *bundle)
 {
